@@ -6,17 +6,19 @@ use Dancer::Config;
 use DBI;
 use Dancer::Plugin::Database::Handle;
 
+=encoding utf8
+
 =head1 NAME
 
 Dancer::Plugin::Database - easy database connections for Dancer applications
 
 =cut
 
-our $VERSION = '1.42_01';
+our $VERSION = '1.60_01';
 
 my $settings = undef;
 
-sub _load_db_settings { $settings = plugin_setting; }
+sub _load_db_settings { $settings = plugin_setting(); }
 
 my %handles;
 # Hashref used as key for default handle, so we don't have a magic value that
@@ -95,6 +97,8 @@ register database => sub {
     }
 };
 
+Dancer::Factory::Hook->instance->install_hooks(qw(database_connected));
+
 register_plugin;
 
 # Given the settings to use, try to get a database connection
@@ -102,8 +106,8 @@ sub _get_connection {
     my $settings = shift;
 
     # Assemble the DSN:
-    my $dsn;
-    my $driver;
+    my $dsn = '';
+    my $driver = '';
     if ($settings->{dsn}) {
         $dsn = $settings->{dsn};
         ($driver) = $dsn =~ m{dbi:([^:]+)};
@@ -116,8 +120,10 @@ sub _get_connection {
         # (DBI's documentation recommends that DBD::* modules should understand
         # 'database', but older versions of DBD::SQLite didn't; let's make 
         # things easier for our users by handling this for them):
-        # (DBD::SQLite will support 'database', too, as of 1.32 when it's
-        # released)
+        # (I asked in RT #61117 for DBD::SQLite to support 'database', too; this
+        # was included in DBD::SQLite 1.33, released Mon 20 May 2011.
+        # Special-casing may as well stay, rather than forcing dependency on
+        # DBD::SQLite 1.33.
         if ($driver eq 'SQLite' 
             && $settings->{database} && !$settings->{dbname}) {
             $settings->{dbname} = delete $settings->{database};
@@ -173,6 +179,8 @@ sub _get_connection {
         }
     }
 
+    Dancer::Factory::Hook->instance->execute_hooks('database_connected', $dbh);
+
     # Indicate whether queries generated by quick_query() etc in
     # Dancer::Plugin::Database::Handle should be logged or not; this seemed a
     # little dirty, but DBI's docs encourage it
@@ -234,7 +242,7 @@ sub _get_settings {
         }
     }
 
-    # We should have soemthing to return now; make sure we have a
+    # We should have something to return now; make sure we have a
     # connection_check_threshold, then return what we found.  In previous
     # versions the documentation contained a typo mentioning
     # connectivity-check-threshold, so support that as an alias.
@@ -334,7 +342,7 @@ handles cases where the database handle hasn't been used for a while and the
 underlying connection has gone away.
 
 The C<dbi_params> setting is also optional, and if specified, should be settings
-which can be passed to C<< DBI->connect >> as its third argument; see the L<DBI>
+which can be passed to C<< DBI->connect >> as its fourth argument; see the L<DBI>
 documentation for these.
 
 The optional C<on_connect_do> setting is an array of queries which should be
@@ -444,6 +452,35 @@ Examples:
   # Fetch all rows from a table (since version 1.30):
   database->quick_select($table_name, {});
 
+There's more extensive documentation on these features in
+L<Dancer::Plugin::Database::Handle>, including using the C<order_by>, C<limit>,
+C<columns> options to sort / limit results and include only specific columns.
+
+=head1 HOOKS
+
+This plugin uses Dancer's hooks support to allow you to register code that
+should execute at given times - for example:
+
+    hook 'database_connected' => sub {
+        my $dbh = shift;
+        # do something with the new DB handle here
+    };
+
+Currrently defined hook positions are:
+
+=over 4
+
+=item C<database_connected>
+
+Called when a new database connection has been established, after performing any
+C<on_connect_do> statements, but before the handle is returned.  Receives the
+new database handle as a parameter, so that you can do what you need with it.
+
+=back
+
+If you need other hook positions which would be useful to you, please feel free
+to suggest them!
+
 
 =head1 AUTHOR
 
@@ -491,6 +528,15 @@ Alberto Simões
 
 James Aitken (LoonyPandora)
 
+Mark Allen (mrallen1)
+
+Sergiy Borodych (bor)
+
+Mario Domgoergen (mdom)
+
+Andrey Inishev (inish777)
+
+
 =head1 BUGS
 
 Please report any bugs or feature requests to C<bug-dancer-plugin-database at rt.cpan.org>, or through
@@ -534,7 +580,7 @@ You can find the author on IRC in the channel C<#dancer> on <irc.perl.org>.
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2010-11 David Precious.
+Copyright 2010-12 David Precious.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
